import mythic_container
from mythic_container.logging import logger
from mythic_container.utils_mythic_file_transfer import sendFileToMythic

MYTHIC_RPC_FILE_CREATE = "mythic_rpc_file_create"


class MythicRPCFileCreateMessage:
    """
    Must supply TaskID, PayloadUUID, or AgentCallbackID in order to properly assign the file to your operation
    """
    def __init__(self,
                 TaskID: int = 0,
                 PayloadUUID: str = "",
                 AgentCallbackID: str = "",
                 FileContents: bytes = None,
                 DeleteAfterFetch: bool = False,
                 Filename: str = None,
                 IsScreenshot: bool = False,
                 IsDownloadFromAgent: bool = False,
                 RemotePathOnTarget: str = "",
                 TargetHostName: str = "",
                 Comment: str = "",
                 **kwargs):
        self.TaskID = TaskID
        self.PayloadUUID = PayloadUUID
        self.AgentCallbackID = AgentCallbackID
        self.FileContents = FileContents
        self.DeleteAfterFetch = DeleteAfterFetch
        self.Filename = Filename
        self.IsScreenshot = IsScreenshot
        self.IsDownloadFromAgent = IsDownloadFromAgent
        self.RemotePathOnTarget = RemotePathOnTarget
        self.TargetHostName = TargetHostName
        self.Comment = Comment
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "payload_uuid": self.PayloadUUID,
            "agent_callback_id": self.AgentCallbackID,
            "delete_after_fetch": self.DeleteAfterFetch,
            "filename": self.Filename,
            "is_screenshot": self.IsScreenshot,
            "is_download": self.IsDownloadFromAgent,
            "remote_path": self.RemotePathOnTarget,
            "host": self.TargetHostName,
            "comment": self.Comment
        }


class MythicRPCFileCreateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 agent_file_id: str = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.AgentFileId = agent_file_id
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCFileCreate(
        msg: MythicRPCFileCreateMessage) -> MythicRPCFileCreateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_FILE_CREATE,
                                                                            body=msg.to_json())
    finalResponse = MythicRPCFileCreateMessageResponse(**response)
    if finalResponse.Success and finalResponse.AgentFileId is not None:
        # we need to ship off the file contents now
        if await sendFileToMythic(msg.FileContents, finalResponse.AgentFileId):
            return finalResponse
        else:
            finalResponse.Success = False
            finalResponse.Error = "Failed to upload file contents to Mythic over HTTP"
            return finalResponse
    return finalResponse



import mythic_container
from mythic_container.logging import logger
import base64

MYTHIC_RPC_CALLBACK_DECRYPT_BYTES = "mythic_rpc_callback_decrypt_bytes"


class MythicRPCCallbackDecryptBytesMessage:
    def __init__(self,
                 AgentCallbackUUID: str,
                 Message: bytes,
                 IncludesUUID: bool = False,
                 IsBase64Encoded: bool = True,
                 **kwargs):
        self.AgentCallbackUUID = AgentCallbackUUID
        self.Message = Message
        self.IncludesUUID = IncludesUUID
        self.IsBase64Encoded = IsBase64Encoded
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "agent_callback_id": self.AgentCallbackUUID,
            "message": base64.b64encode(self.Message).decode(),
            "include_uuid": self.IncludesUUID,
            "base64_message": self.IsBase64Encoded
        }


class MythicRPCCallbackDecryptBytesMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 message: bytes = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.Message = base64.b64decode(message)
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCCallbackDecryptBytes(
        msg: MythicRPCCallbackDecryptBytesMessage) -> MythicRPCCallbackDecryptBytesMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_CALLBACK_DECRYPT_BYTES,
                                                                            body=msg.to_json())
    return MythicRPCCallbackDecryptBytesMessageResponse(**response)



#!/usr/bin/env python3
import sys
import json
import ujson
import os
import traceback
import pathlib
import asyncio
import mythic_container
from . import MythicCommandBase
from . import PayloadBuilder
from .logging import logger, initialize
from . import agent_utils
from . import MythicGoRPC
from . import C2ProfileBase
from . import c2_utils
from . import TranslationBase
from . import WebhookBase
from . import LoggingBase
from . import webhook_utils
from . import logging_utils
from . import AuthBase
from . import auth_utils
from . import EventingBase
from . import eventing_utils
from .rabbitmq import failedConnectRetryDelay


# start our service
def start_and_run_forever():
    try:
        loop = asyncio.get_event_loop()
        loop.run_until_complete(start_services())
        loop.run_forever()
        logger.error("start_and_run_forever finished")
    except KeyboardInterrupt:
        sys.exit(0)


def getRoutingKey(containerName: str, baseKey: str) -> str:
    return f"{containerName}_{baseKey}"


def getWebhookRoutingKey(webhookType: str) -> str:
    return f"{mythic_container.EMIT_WEBHOOK_ROUTING_KEY_PREFIX}.{webhookType}"


def getLoggingRoutingKey(loggingType: str) -> str:
    return f"{mythic_container.EMIT_LOG_ROUTING_KEY_PREFIX}.{loggingType}"


# Shared file based functionality for all services
async def listFile(msg: bytes) -> bytes:
    try:
        msgDict = ujson.loads(msg)
        for name, c2 in mythic_container.C2ProfileBase.c2Profiles.items():
            if c2.name == msgDict["container_name"]:
                response = listFilesOfPath(c2.server_folder_path)
                return ujson.dumps(response.to_json()).encode()
        # if it's not a c2 profile, then just use the current path of the script that's executing
        response = listFilesOfPath(os.path.dirname(os.path.abspath(sys.argv[0])))
        return ujson.dumps(response.to_json()).encode()
    except Exception as e:
        response = mythic_container.SharedClasses.ListFileMessageResponse(
            Success=False,
            Error=f"Hit exception trying to call list file function function: {traceback.format_exc()}\n{e}"
        )
        return ujson.dumps(response.to_json()).encode()


def listFilesOfPath(path: str) -> mythic_container.SharedClasses.ListFileMessageResponse:
    response = mythic_container.SharedClasses.ListFileMessageResponse(Success=False)
    try:
        files = os.listdir(path)
        files = [f for f in files if os.path.isfile(pathlib.Path(path) / f)]
        response.Files = files
        response.Success = True
    except Exception as e:
        response.Error = f"{traceback.format_exc()}\n{e}"
    return response


async def removeFile(msg: bytes) -> bytes:
    try:
        msgDict = ujson.loads(msg)
        inputMsg = mythic_container.SharedClasses.RemoveFileMessage(**msgDict)
        for name, c2 in mythic_container.C2ProfileBase.c2Profiles.items():
            if c2.name == msgDict["container_name"]:
                response = removeFileOfPath(c2.server_folder_path / inputMsg.Filename)
                return ujson.dumps(response.to_json()).encode()
        path = pathlib.Path(os.path.dirname(os.path.abspath(sys.argv[0]))) / inputMsg.Filename
        response = removeFileOfPath(path)
        return ujson.dumps(response.to_json()).encode()
    except Exception as e:
        response = mythic_container.SharedClasses.GetFileMessageResponse(
            Success=False,
            Error=f"Hit exception trying to call get removeFile function: {traceback.format_exc()}\n{e}"
        )
        return ujson.dumps(response.to_json()).encode()


def removeFileOfPath(path: str) -> mythic_container.SharedClasses.RemoveFileMessageResponse:
    response = mythic_container.SharedClasses.RemoveFileMessageResponse(Success=False)
    try:
        path = path.resolve()
        os.remove(path)
        response.Success = True
    except Exception as e:
        response.Error = f"{traceback.format_exc()}\n{e}"
    return response


async def getFile(msg: bytes) -> bytes:
    try:
        msgDict = ujson.loads(msg)
        inputMsg = mythic_container.SharedClasses.GetFileMessage(**msgDict)
        for name, c2 in mythic_container.C2ProfileBase.c2Profiles.items():
            if c2.name == msgDict["container_name"]:
                response = getFileOfPath(c2.server_folder_path / inputMsg.Filename)
                return ujson.dumps(response.to_json()).encode()
        path = pathlib.Path(os.path.dirname(os.path.abspath(sys.argv[0]))) / inputMsg.Filename
        response = getFileOfPath(path)
        return ujson.dumps(response.to_json()).encode()
    except Exception as e:
        response = mythic_container.SharedClasses.GetFileMessageResponse(
            Success=False,
            Error=f"Hit exception trying to call get file function function: {traceback.format_exc()}\n{e}"
        )
        return ujson.dumps(response.to_json()).encode()


def getFileOfPath(path: str) -> mythic_container.SharedClasses.GetFileMessageResponse:
    response = mythic_container.SharedClasses.GetFileMessageResponse(Success=False)
    try:
        file_data = open(path, "rb").read()
        response.Success = True
        response.Message = file_data
    except Exception as e:
        response.Error = f"{traceback.format_exc()}\n{e}"
    return response


async def writeFile(msg: bytes) -> bytes:
    try:
        msgDict = ujson.loads(msg)
        inputMsg = mythic_container.SharedClasses.WriteFileMessage(**msgDict)
        for name, c2 in mythic_container.C2ProfileBase.c2Profiles.items():
            if c2.name == msgDict["container_name"]:
                response = writeFileOfPath(c2.server_folder_path / inputMsg.Filename, inputMsg.Contents)
                return ujson.dumps(response.to_json()).encode()
        path = pathlib.Path(os.path.dirname(os.path.abspath(sys.argv[0]))) / inputMsg.Filename
        response = writeFileOfPath(path, inputMsg.Contents)
        return ujson.dumps(response.to_json()).encode()
    except Exception as e:
        logger.exception(f"[-] Failed to write to file with exception: {e}")
        response = mythic_container.SharedClasses.WriteFileMessageResponse(
            Success=False,
            Error=f"Hit exception trying to call write file function function: {traceback.format_exc()}\n{e}"
        )
        return ujson.dumps(response.to_json()).encode()


def writeFileOfPath(path: str, bytesToWrite: bytes) -> mythic_container.SharedClasses.WriteFileMessageResponse:
    response = mythic_container.SharedClasses.WriteFileMessageResponse(Success=False)
    try:
        with open(path, "wb") as f:
            f.write(bytesToWrite)
        response.Success = True
        response.Message = "Successfully wrote file"
    except Exception as e:
        logger.exception(f"[-] Failed to write to file: {e}")
        response.Error = f"{traceback.format_exc()}\n{e}"
    return response


async def onStart(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        inputMsg = mythic_container.SharedClasses.ContainerOnStartMessage(**msgDict)
        for name, c2 in mythic_container.C2ProfileBase.c2Profiles.items():
            if c2.name == inputMsg.ContainerName:
                response = await c2.on_container_start(inputMsg)
                await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                    queue=mythic_container.CONTAINER_ON_START_RESPONSE,
                    body=response.to_json()
                )
                return
        for name, pt in mythic_container.PayloadBuilder.payloadTypes.items():
            if pt.name == inputMsg.ContainerName:
                response = await pt.on_container_start(inputMsg)
                await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                    queue=mythic_container.CONTAINER_ON_START_RESPONSE,
                    body=response.to_json()
                )
                return
        for name, tr in mythic_container.TranslationBase.translationServices.items():
            if tr.name == inputMsg.ContainerName:
                response = await tr.on_container_start(inputMsg)
                await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                    queue=mythic_container.CONTAINER_ON_START_RESPONSE,
                    body=response.to_json()
                )
                return
        for name, lg in mythic_container.LoggingBase.loggers.items():
            if lg.name == inputMsg.ContainerName:
                response = await lg.on_container_start(inputMsg)
                await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                    queue=mythic_container.CONTAINER_ON_START_RESPONSE,
                    body=response.to_json()
                )
                return
        for name, wb in mythic_container.WebhookBase.webhooks.items():
            if wb.name == inputMsg.ContainerName:
                response = await wb.on_container_start(inputMsg)
                await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                    queue=mythic_container.CONTAINER_ON_START_RESPONSE,
                    body=response.to_json()
                )
                return
        for name, auth in mythic_container.AuthBase.authServices.items():
            if auth.name == inputMsg.ContainerName:
                response = await auth.on_container_start(inputMsg)
                await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                    queue=mythic_container.CONTAINER_ON_START_RESPONSE,
                    body=response.to_json()
                )
                return
        for name, event in mythic_container.EventingBase.eventingServices.items():
            if event.name == inputMsg.ContainerName:
                response = await event.on_container_start(inputMsg)
                await mythic_container.RabbitmqConnection.SendDictDirectMessage(
                    queue=mythic_container.CONTAINER_ON_START_RESPONSE,
                    body=response.to_json()
                )
                return
    except Exception as e:
        logger.exception(f"[-] Failed to call container on start with exception: {e}")
        return

payloadQueueTasks = []


async def startPayloadRabbitMQ(pt: PayloadBuilder.PayloadType) -> None:
    await startSharedServices(pt.name)
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(pt.name, mythic_container.PAYLOAD_BUILD_ROUTING_KEY),
        routing_key=getRoutingKey(pt.name, mythic_container.PAYLOAD_BUILD_ROUTING_KEY),
        handler=agent_utils.buildWrapper
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(pt.name, mythic_container.PT_ON_NEW_CALLBACK_ROUTING_KEY),
        routing_key=getRoutingKey(pt.name, mythic_container.PT_ON_NEW_CALLBACK_ROUTING_KEY),
        handler=agent_utils.onNewCallback
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(pt.name, mythic_container.PT_TASK_OPSEC_PRE_CHECK),
        routing_key=getRoutingKey(pt.name, mythic_container.PT_TASK_OPSEC_PRE_CHECK),
        handler=agent_utils.opsecPreCheck
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(pt.name, mythic_container.PT_TASK_CREATE_TASKING),
        routing_key=getRoutingKey(pt.name, mythic_container.PT_TASK_CREATE_TASKING),
        handler=agent_utils.createTasking
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(pt.name, mythic_container.PT_TASK_OPSEC_POST_CHECK),
        routing_key=getRoutingKey(pt.name, mythic_container.PT_TASK_OPSEC_POST_CHECK),
        handler=agent_utils.opsecPostCheck
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(pt.name, mythic_container.PT_TASK_PROCESS_RESPONSE),
        routing_key=getRoutingKey(pt.name, mythic_container.PT_TASK_PROCESS_RESPONSE),
        handler=agent_utils.processResponse
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(pt.name, mythic_container.PT_TASK_COMPLETION_FUNCTION),
        routing_key=getRoutingKey(pt.name, mythic_container.PT_TASK_COMPLETION_FUNCTION),
        handler=agent_utils.completionFunction
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(pt.name, mythic_container.PT_RPC_COMMAND_DYNAMIC_QUERY_FUNCTION),
        routing_key=getRoutingKey(pt.name, mythic_container.PT_RPC_COMMAND_DYNAMIC_QUERY_FUNCTION),
        handler=agent_utils.dynamicQueryFunction
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(pt.name, mythic_container.PT_RPC_COMMAND_TYPEDARRAY_PARSE_FUNCTION),
        routing_key=getRoutingKey(pt.name, mythic_container.PT_RPC_COMMAND_TYPEDARRAY_PARSE_FUNCTION),
        handler=agent_utils.typedTaskParseFunction
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(pt.name, mythic_container.MYTHIC_RPC_OTHER_SERVICES_RPC),
        routing_key=getRoutingKey(pt.name, mythic_container.MYTHIC_RPC_OTHER_SERVICES_RPC),
        handler=agent_utils.customRPCFunction
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(pt.name, mythic_container.PT_RPC_RESYNC_ROUTING_KEY),
        routing_key=getRoutingKey(pt.name, mythic_container.PT_RPC_RESYNC_ROUTING_KEY),
        handler=agent_utils.reSyncPayloadType
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(pt.name, mythic_container.PT_CHECK_IF_CALLBACKS_ALIVE),
        routing_key=getRoutingKey(pt.name, mythic_container.PT_CHECK_IF_CALLBACKS_ALIVE),
        handler=agent_utils.checkIfCallbacksAlive
    )))


async def syncPayloadData(pt: PayloadBuilder.PayloadType) -> None:
    syncMessage = {
        "payload_type": pt.to_json(),
        "commands": [],
        "container_version": mythic_container.containerVersion
    }
    for cls in MythicCommandBase.CommandBase.__subclasses__():
        if cls.__module__.split(".")[0].lower() == pt.name:
            logger.info(f"[*] Processing command {cls.cmd}")
            if pt.name not in MythicCommandBase.commands:
                MythicCommandBase.commands[pt.name] = []
            MythicCommandBase.commands[pt.name].append(
                cls(pt.agent_path, pt.agent_code_path, pt.agent_browserscript_path))
            syncMessage["commands"].append(
                cls(pt.agent_path, pt.agent_code_path, pt.agent_browserscript_path).to_json())
    while True:
        response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(
            queue=mythic_container.PT_SYNC_ROUTING_KEY,
            body=syncMessage)
        if response is None:
            logger.error("[-] Failed to get a response back from syncing RPC message, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif "success" not in response:
            logger.error("[-] RPC response doesn't contain success, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif not response["success"]:
            logger.error(f"[-] Failed to sync {pt.name}: {response['error']}, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        else:
            logger.info(f"[+] Successfully synced {pt.name}")
            return


async def startC2RabbitMQ(c2: C2ProfileBase.C2Profile) -> None:
    await startSharedServices(c2.name)
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(c2.name, mythic_container.C2_RPC_START_SERVER_ROUTING_KEY),
        routing_key=getRoutingKey(c2.name, mythic_container.C2_RPC_START_SERVER_ROUTING_KEY),
        handler=c2_utils.startServer
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(c2.name, mythic_container.C2_RPC_CONFIG_CHECK_ROUTING_KEY),
        routing_key=getRoutingKey(c2.name, mythic_container.C2_RPC_CONFIG_CHECK_ROUTING_KEY),
        handler=c2_utils.configChecks
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(c2.name, mythic_container.C2_RPC_GET_SERVER_DEBUG_OUTPUT),
        routing_key=getRoutingKey(c2.name, mythic_container.C2_RPC_GET_SERVER_DEBUG_OUTPUT),
        handler=c2_utils.getDebugOutput
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(c2.name, mythic_container.C2_RPC_REDIRECTOR_RULES_ROUTING_KEY),
        routing_key=getRoutingKey(c2.name, mythic_container.C2_RPC_REDIRECTOR_RULES_ROUTING_KEY),
        handler=c2_utils.getRedirectorRules
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(c2.name, mythic_container.C2_RPC_OPSEC_CHECKS_ROUTING_KEY),
        routing_key=getRoutingKey(c2.name, mythic_container.C2_RPC_OPSEC_CHECKS_ROUTING_KEY),
        handler=c2_utils.opsecChecks
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(c2.name, mythic_container.C2_RPC_STOP_SERVER_ROUTING_KEY),
        routing_key=getRoutingKey(c2.name, mythic_container.C2_RPC_STOP_SERVER_ROUTING_KEY),
        handler=c2_utils.stopServer
    )))

    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(c2.name, mythic_container.MYTHIC_RPC_OTHER_SERVICES_RPC),
        routing_key=getRoutingKey(c2.name, mythic_container.MYTHIC_RPC_OTHER_SERVICES_RPC),
        handler=c2_utils.customRPCFunction
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(c2.name, mythic_container.C2_RPC_RESYNC_ROUTING_KEY),
        routing_key=getRoutingKey(c2.name, mythic_container.C2_RPC_RESYNC_ROUTING_KEY),
        handler=c2_utils.reSyncC2Profile
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(c2.name, mythic_container.C2_RPC_GET_IOC_ROUTING_KEY),
        routing_key=getRoutingKey(c2.name, mythic_container.C2_RPC_GET_IOC_ROUTING_KEY),
        handler=c2_utils.getIOC
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(c2.name, mythic_container.C2_RPC_SAMPLE_MESSAGE_ROUTING_KEY),
        routing_key=getRoutingKey(c2.name, mythic_container.C2_RPC_SAMPLE_MESSAGE_ROUTING_KEY),
        handler=c2_utils.sampleMessage
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(c2.name, mythic_container.C2_RPC_HOST_FILE_ROUTING_KEY),
        routing_key=getRoutingKey(c2.name, mythic_container.C2_RPC_HOST_FILE_ROUTING_KEY),
        handler=c2_utils.hostFile
    )))


async def syncC2ProfileData(c2: C2ProfileBase.C2Profile) -> None:
    syncMessage = {
        **c2.to_json(),
        "container_version": mythic_container.containerVersion
    }

    while True:
        response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(
            queue=mythic_container.C2_SYNC_ROUTING_KEY,
            body=syncMessage)
        if response is None:
            logger.error("[-] Failed to get a response back from syncing RPC message, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif "success" not in response:
            logger.error("[-] RPC response doesn't contain success, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif not response["success"]:
            logger.error(f"[-] Failed to sync {c2.name}: {response['error']}, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        else:
            logger.info(f"[+] Successfully synced {c2.name}")
            return


async def startTranslatorRabbitMQ(tr: TranslationBase.TranslationContainer) -> None:
    await startSharedServices(tr.name)
    payloadQueueTasks.append(asyncio.create_task(TranslationBase.handleTranslationServices(tr.name)))


async def syncTranslatorData(tr: TranslationBase.TranslationContainer) -> None:
    syncMessage = {
        **tr.to_json(),
        "container_version": mythic_container.containerVersion
    }
    while True:
        response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(
            queue=mythic_container.TR_SYNC_ROUTING_KEY,
            body=syncMessage)
        if response is None:
            logger.error("[-] Failed to get a response back from syncing RPC message, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif "success" not in response:
            logger.error("[-] RPC response doesn't contain success, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif not response["success"]:
            logger.error(f"[-] Failed to sync {tr.name}: {response['error']}, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        else:
            logger.info(f"[+] Successfully synced {tr.name}")
            return


async def syncWebhookData(wb: WebhookBase.Webhook) -> None:
    syncMessage = {
        "consuming_container": wb.get_sync_message(),
        "container_version": mythic_container.containerVersion
    }
    await startSharedServices(wb.name)
    while True:
        response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(
            queue=mythic_container.CONSUMING_CONTAINER_SYNC_ROUTING_KEY,
            body=syncMessage)
        if response is None:
            logger.error("[-] Failed to get a response back from syncing RPC message, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif "success" not in response:
            logger.error("[-] RPC response doesn't contain success, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif not response["success"]:
            logger.error(f"[-] Failed to sync {wb.name}: {response['error']}, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        else:
            logger.info(f"[+] Successfully synced {wb.name}")
            break

    if wb.new_startup is not None and callable(wb.new_startup):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getWebhookRoutingKey(mythic_container.WEBHOOK_TYPE_NEW_STARTUP),
                routing_key=getWebhookRoutingKey(mythic_container.WEBHOOK_TYPE_NEW_STARTUP),
                handler=webhook_utils.new_startup
            )))
    if wb.new_callback is not None and callable(wb.new_callback):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getWebhookRoutingKey(mythic_container.WEBHOOK_TYPE_NEW_CALLBACK),
                routing_key=getWebhookRoutingKey(mythic_container.WEBHOOK_TYPE_NEW_CALLBACK),
                handler=webhook_utils.new_callback
            )))
    if wb.new_feedback is not None and callable(wb.new_feedback):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getWebhookRoutingKey(mythic_container.WEBHOOK_TYPE_NEW_FEEDBACK),
                routing_key=getWebhookRoutingKey(mythic_container.WEBHOOK_TYPE_NEW_FEEDBACK),
                handler=webhook_utils.new_feedback
            )))
    if wb.new_alert is not None and callable(wb.new_alert):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getWebhookRoutingKey(mythic_container.WEBHOOK_TYPE_NEW_ALERT),
                routing_key=getWebhookRoutingKey(mythic_container.WEBHOOK_TYPE_NEW_ALERT),
                handler=webhook_utils.new_alert
            )))
    if wb.new_custom is not None and callable(wb.new_custom):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getWebhookRoutingKey(mythic_container.WEBHOOK_TYPE_NEW_CUSTOM),
                routing_key=getWebhookRoutingKey(mythic_container.WEBHOOK_TYPE_NEW_CUSTOM),
                handler=webhook_utils.new_custom
            )))
    logger.info(f"Successfully started webhook service")


async def syncLoggingData(wb: LoggingBase.Log) -> None:
    syncMessage = {
        "consuming_container": wb.get_sync_message(),
        "container_version": mythic_container.containerVersion
    }
    await startSharedServices(wb.name)
    while True:
        response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(
            queue=mythic_container.CONSUMING_CONTAINER_SYNC_ROUTING_KEY,
            body=syncMessage)
        if response is None:
            logger.error("[-] Failed to get a response back from syncing RPC message, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif "success" not in response:
            logger.error("[-] RPC response doesn't contain success, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif not response["success"]:
            logger.error(f"[-] Failed to sync {wb.name}: {response['error']}, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        else:
            logger.info(f"[+] Successfully synced {wb.name}")
            break

    if wb.new_callback is not None and callable(wb.new_callback):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getLoggingRoutingKey(mythic_container.LOG_TYPE_CALLBACK),
                routing_key=getLoggingRoutingKey(mythic_container.LOG_TYPE_CALLBACK),
                handler=logging_utils.new_callback
            )))
    if wb.new_credential is not None and callable(wb.new_credential):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getLoggingRoutingKey(mythic_container.LOG_TYPE_CREDENTIAL),
                routing_key=getLoggingRoutingKey(mythic_container.LOG_TYPE_CREDENTIAL),
                handler=logging_utils.new_credential
            )))
    if wb.new_keylog is not None and callable(wb.new_keylog):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getLoggingRoutingKey(mythic_container.LOG_TYPE_KEYLOG),
                routing_key=getLoggingRoutingKey(mythic_container.LOG_TYPE_KEYLOG),
                handler=logging_utils.new_keylog
            )))
    if wb.new_file is not None and callable(wb.new_file):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getLoggingRoutingKey(mythic_container.LOG_TYPE_FILE),
                routing_key=getLoggingRoutingKey(mythic_container.LOG_TYPE_FILE),
                handler=logging_utils.new_file
            )))
    if wb.new_payload is not None and callable(wb.new_payload):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getLoggingRoutingKey(mythic_container.LOG_TYPE_PAYLOAD),
                routing_key=getLoggingRoutingKey(mythic_container.LOG_TYPE_PAYLOAD),
                handler=logging_utils.new_payload
            )))
    if wb.new_artifact is not None and callable(wb.new_artifact):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getLoggingRoutingKey(mythic_container.LOG_TYPE_ARTIFACT),
                routing_key=getLoggingRoutingKey(mythic_container.LOG_TYPE_ARTIFACT),
                handler=logging_utils.new_artifact
            )))
    if wb.new_task is not None and callable(wb.new_task):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getLoggingRoutingKey(mythic_container.LOG_TYPE_TASK),
                routing_key=getLoggingRoutingKey(mythic_container.LOG_TYPE_TASK),
                handler=logging_utils.new_task
            )))
    if wb.new_response is not None and callable(wb.new_response):
        payloadQueueTasks.append(
            asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectTopicExchange(
                queue=getLoggingRoutingKey(mythic_container.LOG_TYPE_RESPONSE),
                routing_key=getLoggingRoutingKey(mythic_container.LOG_TYPE_RESPONSE),
                handler=logging_utils.new_response
            )))

    logger.info(f"Successfully started logging service")


async def syncAuthData(wb: AuthBase.Auth) -> None:
    syncMessage = {
        "consuming_container": wb.get_sync_message(),
        "container_version": mythic_container.containerVersion
    }
    await startSharedServices(wb.name)
    while True:
        response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(
            queue=mythic_container.CONSUMING_CONTAINER_SYNC_ROUTING_KEY,
            body=syncMessage)
        if response is None:
            logger.error("[-] Failed to get a response back from syncing RPC message, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif "success" not in response:
            logger.error("[-] RPC response doesn't contain success, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif not response["success"]:
            logger.error(f"[-] Failed to sync {wb.name}: {response['error']}, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        else:
            logger.info(f"[+] Successfully synced {wb.name}")
            break

    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(wb.name, mythic_container.AUTH_RPC_GET_IDP_METADATA),
        routing_key=getRoutingKey(wb.name, mythic_container.AUTH_RPC_GET_IDP_METADATA),
        handler=auth_utils.GetIDPMetadata
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(wb.name, mythic_container.AUTH_RPC_GET_IDP_REDIRECT),
        routing_key=getRoutingKey(wb.name, mythic_container.AUTH_RPC_GET_IDP_REDIRECT),
        handler=auth_utils.GetIDPRedirect
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(wb.name, mythic_container.AUTH_RPC_PROCESS_IDP_RESPONSE),
        routing_key=getRoutingKey(wb.name, mythic_container.AUTH_RPC_PROCESS_IDP_RESPONSE),
        handler=auth_utils.ProcessIDPResponse
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(wb.name, mythic_container.AUTH_RPC_GET_NONIDP_METADATA),
        routing_key=getRoutingKey(wb.name, mythic_container.AUTH_RPC_GET_NONIDP_METADATA),
        handler=auth_utils.GetNonIDPMetadata
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(wb.name, mythic_container.AUTH_RPC_GET_NONIDP_REDIRECT),
        routing_key=getRoutingKey(wb.name, mythic_container.AUTH_RPC_GET_NONIDP_REDIRECT),
        handler=auth_utils.GetNonIDPRedirect
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(wb.name, mythic_container.AUTH_RPC_PROCESS_NONIDP_RESPONSE),
        routing_key=getRoutingKey(wb.name, mythic_container.AUTH_RPC_PROCESS_NONIDP_RESPONSE),
        handler=auth_utils.ProcessNonIDPResponse
    )))
    logger.info(f"Successfully started auth service")


async def syncEventingData(wb: EventingBase.Eventing) -> None:
    syncMessage = {
        "consuming_container": wb.get_sync_message(),
        "container_version": mythic_container.containerVersion
    }
    await startSharedServices(wb.name)
    while True:
        response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(
            queue=mythic_container.CONSUMING_CONTAINER_SYNC_ROUTING_KEY,
            body=syncMessage)
        if response is None:
            logger.error("[-] Failed to get a response back from syncing RPC message, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif "success" not in response:
            logger.error("[-] RPC response doesn't contain success, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        elif not response["success"]:
            logger.error(f"[-] Failed to sync {wb.name}: {response['error']}, trying again...")
            await asyncio.sleep(failedConnectRetryDelay)
        else:
            logger.info(f"[+] Successfully synced {wb.name}")
            break

    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(wb.name, mythic_container.EVENTING_TASK_INTERCEPT),
        routing_key=getRoutingKey(wb.name, mythic_container.EVENTING_TASK_INTERCEPT),
        handler=eventing_utils.TaskIntercept
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(wb.name, mythic_container.EVENTING_RESPONSE_INTERCEPT),
        routing_key=getRoutingKey(wb.name, mythic_container.EVENTING_RESPONSE_INTERCEPT),
        handler=eventing_utils.ResponseIntercept
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(wb.name, mythic_container.EVENTING_CUSTOM_FUNCTION),
        routing_key=getRoutingKey(wb.name, mythic_container.EVENTING_CUSTOM_FUNCTION),
        handler=eventing_utils.CustomFunction
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(wb.name, mythic_container.EVENTING_CONDITIONAL_CHECK),
        routing_key=getRoutingKey(wb.name, mythic_container.EVENTING_CONDITIONAL_CHECK),
        handler=eventing_utils.ConditionalEventingCheck
    )))
    logger.info(f"Successfully started Eventing service")


async def startSharedServices(containerName: str):
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(containerName, mythic_container.CONTAINER_RPC_GET_FILE),
        routing_key=getRoutingKey(containerName, mythic_container.CONTAINER_RPC_GET_FILE),
        handler=getFile
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(containerName, mythic_container.CONTAINER_RPC_LIST_FILE),
        routing_key=getRoutingKey(containerName, mythic_container.CONTAINER_RPC_LIST_FILE),
        handler=listFile
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(containerName, mythic_container.CONTAINER_RPC_REMOVE_FILE),
        routing_key=getRoutingKey(containerName, mythic_container.CONTAINER_RPC_REMOVE_FILE),
        handler=removeFile
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromRPCQueue(
        queue=getRoutingKey(containerName, mythic_container.CONTAINER_RPC_WRITE_FILE),
        routing_key=getRoutingKey(containerName, mythic_container.CONTAINER_RPC_WRITE_FILE),
        handler=writeFile
    )))
    payloadQueueTasks.append(asyncio.create_task(mythic_container.RabbitmqConnection.ReceiveFromMythicDirectExchange(
        queue=getRoutingKey(containerName, mythic_container.CONTAINER_ON_START),
        routing_key=getRoutingKey(containerName, mythic_container.CONTAINER_ON_START),
        handler=onStart
    )))


async def start_services():
    initialize()
    logger.info(
        f"[+] Starting Services with version {mythic_container.containerVersion} and PyPi version {mythic_container.PyPi_version}\n")
    webhook_services = WebhookBase.Webhook.__subclasses__()
    for cls in webhook_services:
        logger.info(f"[*] Processing webhook service")
        webhook = cls()
        if webhook.name == "":
            logger.error("missing name for webhook")
            continue
        if webhook.name in WebhookBase.webhooks:
            logger.error(f"[-] attempting to import {webhook.name} multiple times - probably due to import issues")
            continue
        WebhookBase.webhooks[webhook.name] = webhook
        await syncWebhookData(webhook)
    logging_services = LoggingBase.Log.__subclasses__()
    for cls in logging_services:
        logger.info(f"[*] Processing logging services")
        definedLog = cls()
        if definedLog.name == "":
            logger.error("missing name for logger")
            continue
        if definedLog.name in LoggingBase.loggers:
            logger.error(f"[-] attempting to import {definedLog.name} multiple times - probably due to import issues")
            continue
        LoggingBase.loggers[definedLog.name] = definedLog
        await syncLoggingData(definedLog)
    payloadTypes = PayloadBuilder.PayloadType.__subclasses__()
    for cls in payloadTypes:
        payload_type = cls()
        if payload_type.name == "":
            logger.error("missing name for payload_type")
            continue
        if payload_type.name in PayloadBuilder.payloadTypes:
            logger.error(f"[-] attempting to import {payload_type.name} multiple times - probably due to import issues")
            continue
        PayloadBuilder.payloadTypes[payload_type.name] = payload_type
        logger.info(f"[*] Processing agent: {payload_type.name}")
        await syncPayloadData(payload_type)
        await startPayloadRabbitMQ(payload_type)

    c2Profiles = C2ProfileBase.C2Profile.__subclasses__()
    for cls in c2Profiles:
        c2profile = cls()
        if c2profile.name == "":
            logger.error("missing name for c2profile")
            continue
        if c2profile.name in C2ProfileBase.c2Profiles:
            logger.error(f"[-] attempting to import {c2profile.name} multiple times - probably due to import issues")
            continue
        C2ProfileBase.c2Profiles[c2profile.name] = c2profile
        logger.info(f"[*] Processing c2 profile: {c2profile.name}")
        await syncC2ProfileData(c2profile)
        await startC2RabbitMQ(c2profile)

    translation_services = TranslationBase.TranslationContainer.__subclasses__()
    for cls in translation_services:
        translator = cls()
        if translator.name == "":
            logger.error("missing name for translator")
            continue
        if translator.name in TranslationBase.translationServices:
            logger.error(f"[-] attempting to import {translator.name} multiple times - probably due to import issues")
            continue
        TranslationBase.translationServices[translator.name] = translator
        logger.info(f"[*] Processing translation service: {translator.name}")
        await syncTranslatorData(translator)
        await startTranslatorRabbitMQ(translator)
    auth_services = AuthBase.Auth.__subclasses__()
    for cls in auth_services:
        auth = cls()
        if auth.name == "":
            logger.error("missing name for auth service")
            continue
        if auth.name in AuthBase.authServices:
            logger.error(f"[-] attempting to import {auth.name} multiple times - probably due to import issues")
            continue
        AuthBase.authServices[auth.name] = auth
        logger.info(f"[*] Processing auth service: {auth.name}")
        await syncAuthData(auth)
    eventing_services = EventingBase.Eventing.__subclasses__()
    for cls in eventing_services:
        event = cls()
        if event.name == "":
            logger.error("missing name for event service")
            continue
        if event.name in EventingBase.eventingServices:
            logger.error(f"[-] attempting to import {event.name} multiple times - probably due to import issues")
            continue
        EventingBase.eventingServices[event.name] = event
        logger.info(f"[*] Processing eventing service: {event.name}")
        await syncEventingData(event)

    logger.info("[+] All services synced with Mythic!")
    logger.info("[*] Starting services to listen...")


async def test_command(payload_type_name: str,
                       command_name: str,
                       operation_name: str = None,
                       task_id: int = None,
                       tasking_location: str = "command_line",
                       parameters_string: str = None,
                       parameters_dictionary: dict = None):
    logger.info(f"[*] Started testing {payload_type_name}'s {command_name} command")
    if operation_name is None or task_id is None:
        logger.info(f"[*] Specify an operation_name and task_id to get real data for testing."
                    f"\nThis does not adjust your specified command/parameters, but allows MythicRPC calls to work properly")
    params = parameters_string
    if parameters_dictionary is not None:
        params = json.dumps(parameters_dictionary)
    payloadTypes = PayloadBuilder.PayloadType.__subclasses__()
    for cls in payloadTypes:
        payload_type = cls()
        if payload_type.name == payload_type_name:
            logger.info(f"[+] Found payload type: {payload_type.name}")
            for cmdcls in MythicCommandBase.CommandBase.__subclasses__():
                if cmdcls.__module__.split(".")[0] == payload_type.name:
                    if cmdcls.cmd == command_name:
                        commandInstance = cmdcls(payload_type.agent_path, payload_type.agent_code_path,
                                                 payload_type.agent_browserscript_path)
                        logger.info(f"[+] Found command: {commandInstance.cmd}")
                        opsecPre = MythicCommandBase.PTTaskMessageAllData(
                            task={
                                "tasking_location": tasking_location,
                                "params": params,
                                "original_params": params
                            },
                            payload_type=payload_type_name,
                            args=cmdcls.argument_class
                        )
                        opsecPost = MythicCommandBase.PTTaskMessageAllData(
                            task={
                                "tasking_location": tasking_location,
                                "params": params,
                                "original_params": params
                            },
                            args=cmdcls.argument_class
                        )
                        createTasking = MythicCommandBase.PTTaskMessageAllData(
                            task={
                                "tasking_location": tasking_location,
                                "params": params,
                                "original_params": params
                            },
                            args=cmdcls.argument_class
                        )
                        if operation_name is None or task_id is None:
                            logger.info(
                                f"[*] operation_name is None, testing with fake data. Some MythicRPC functions might not work")
                        else:
                            logger.info(f"[*] Fetching information for task {task_id} of operation {operation_name}")
                            fetchResp = await MythicGoRPC.SendMythicRPCTaskDisplayToRealIdSearch(
                                MythicGoRPC.MythicRPCTaskDisplayToRealIdSearchMessage(
                                    TaskDisplayID=task_id,
                                    OperationName=operation_name
                                ))
                            if not fetchResp.Success:
                                logger.error(f"[-] Failed to find task: {fetchResp.Error}")
                                sys.exit(1)
                            else:
                                taskResp = await MythicGoRPC.SendMythicRPCTaskSearch(
                                    MythicGoRPC.MythicRPCTaskSearchMessage(
                                        TaskID=fetchResp.TaskID,
                                    ))
                                if not taskResp.Success:
                                    logger.error(f"[-] Failed to get task information: {taskResp.Error}")
                                    sys.exit(1)
                                elif len(taskResp.Tasks) == 0:
                                    logger.error(f"[-] Failed to search for task information")
                                    sys.exit(1)
                                else:
                                    opsecPre.Task = MythicCommandBase.PTTaskMessageTaskData(
                                        **taskResp.Tasks[0].to_json())
                                    opsecPre.Task.CommandName = command_name
                                    opsecPre.Task.Params = params
                                    opsecPre.Task.OriginalParams = params
                                    opsecPre.Task.TaskingLocation = tasking_location
                                    opsecPost.Task = MythicCommandBase.PTTaskMessageTaskData(
                                        **taskResp.Tasks[0].to_json())
                                    opsecPost.Task.CommandName = command_name
                                    opsecPost.Task.Params = params
                                    opsecPost.Task.OriginalParams = params
                                    opsecPost.Task.TaskingLocation = tasking_location
                                    createTasking.Task = MythicCommandBase.PTTaskMessageTaskData(
                                        **taskResp.Tasks[0].to_json())
                                    createTasking.Task.CommandName = command_name
                                    createTasking.Task.Params = params
                                    createTasking.Task.OriginalParams = params
                                    createTasking.Task.TaskingLocation = tasking_location
                        logger.info(f"[*] Testing OPSEC PRE")
                        if not await agent_utils.verifyTaskArgs(opsecPre, ""):
                            return
                        else:
                            try:
                                response = await commandInstance.opsec_pre(taskData=opsecPre)
                                logger.info(f"[+] Finished OPSEC PRE:\n{json.dumps(response.to_json(), indent=4)}")
                            except Exception as e:
                                logger.exception(f"[*] Hit exception: {e}")
                        logger.info(f"[*] Testing Create Tasking")
                        task = await agent_utils.initialize_task(commandInstance, {
                            "task": createTasking.Task.to_json(),
                            "callback": createTasking.Callback.to_json()
                        }, "")
                        if task is None:
                            # we hit an error and already sent the response, just return
                            return
                        else:
                            try:
                                if hasattr(commandInstance, "create_go_tasking"):
                                    if not await agent_utils.verifyTaskArgs(createTasking, ""):
                                        return
                                    createTaskingResponse = await commandInstance.create_go_tasking(
                                        taskData=createTasking)
                                    if createTaskingResponse.Params is None:
                                        # no manual args were set, so parse them from the task.args
                                        createTaskingResponse.Params = str(task.args)
                                    logger.info(
                                        f"[+] Finished Create Tasking (new):\n{json.dumps(createTaskingResponse.to_json(), indent=4)}")
                                else:
                                    createTaskingResponse = await commandInstance.create_tasking(task=task)
                                    logger.info(
                                        f"[+] Finished Create Tasking (legacy):\n{json.dumps(createTaskingResponse.to_json(), indent=4)}")
                            except Exception as createTaskingException:
                                logger.exception(f"[*] Hit exception: {createTaskingException}")
                        logger.info(f"[*] Testing OPSEC Post")
                        if not await agent_utils.verifyTaskArgs(opsecPost, ""):
                            return
                        else:
                            try:
                                response = await commandInstance.opsec_post(taskData=opsecPost)
                                logger.info(f"[+] Finished OPSEC POST:\n{json.dumps(response.to_json(), indent=4)}")
                            except Exception as e:
                                logger.exception(f"[*] Hit exception: {e}")
                        return
            logger.error(f"[-] Failed to find command: {command_name}")
    logger.error(f"[-] Failed to find payload type: {payload_type_name}")



from .rabbitmq import rabbitmqConnectionClass
from .mythic_service import start_and_run_forever, test_command
from .SharedClasses import *

containerVersion = "v1.3.0"

PyPi_version = "0.5.0"

RabbitmqConnection = rabbitmqConnectionClass()

MYTHIC_RPC_OTHER_SERVICES_RPC = "mythic_rpc_other_service_rpc"
PAYLOAD_BUILD_C2_ROUTING_KEY = "payload_c2_build"
# payload routes
PT_SYNC_ROUTING_KEY = "pt_sync"
PT_RPC_RESYNC_ROUTING_KEY = "pt_rpc_resync"
PT_RPC_COMMAND_DYNAMIC_QUERY_FUNCTION = "pt_command_dynamic_query_function"
PT_RPC_COMMAND_TYPEDARRAY_PARSE_FUNCTION = "pt_command_typedarray_parse"
PAYLOAD_BUILD_ROUTING_KEY = "payload_build"
PT_BUILD_RESPONSE_ROUTING_KEY = "pt_build_response"
PT_ON_NEW_CALLBACK_ROUTING_KEY = "pt_on_new_callback"
PT_CHECK_IF_CALLBACKS_ALIVE = "pt_check_if_callbacks_alive"
PT_ON_NEW_CALLBACK_RESPONSE_ROUTING_KEY = "pt_on_new_callback_response"
PT_BUILD_C2_RESPONSE_ROUTING_KEY = "pt_c2_build_response"
PT_TASK_OPSEC_PRE_CHECK = "pt_task_opsec_pre_check"
PT_TASK_OPSEC_PRE_CHECK_RESPONSE = "pt_task_opsec_pre_check_response"
PT_TASK_CREATE_TASKING = "pt_task_create_tasking"
PT_TASK_CREATE_TASKING_RESPONSE = "pt_task_create_tasking_response"
PT_TASK_OPSEC_POST_CHECK = "pt_task_opsec_post_check"
PT_TASK_OPSEC_POST_CHECK_RESPONSE = "pt_task_opsec_post_check_response"
PT_TASK_COMPLETION_FUNCTION = "pt_task_completion_function"
PT_TASK_COMPLETION_FUNCTION_RESPONSE = "pt_task_completion_function_response"
PT_TASK_PROCESS_RESPONSE = "pt_task_process_response"
PT_TASK_PROCESS_RESPONSE_RESPONSE = "pt_task_process_response_response"
# c2 routes
C2_SYNC_ROUTING_KEY = "c2_sync"
C2_RPC_RESYNC_ROUTING_KEY = "c2_rpc_resync"
C2_RPC_OPSEC_CHECKS_ROUTING_KEY = "c2_rpc_opsec_check"
C2_RPC_CONFIG_CHECK_ROUTING_KEY = "c2_rpc_config_check"
C2_RPC_GET_IOC_ROUTING_KEY = "c2_rpc_get_ioc"
C2_RPC_SAMPLE_MESSAGE_ROUTING_KEY = "c2_rpc_sample_message"
C2_RPC_REDIRECTOR_RULES_ROUTING_KEY = "c2_rpc_redirector_rules"
C2_RPC_START_SERVER_ROUTING_KEY = "c2_rpc_start_server"
C2_RPC_STOP_SERVER_ROUTING_KEY = "c2_rpc_stop_server"
C2_RPC_GET_SERVER_DEBUG_OUTPUT = "c2_rpc_get_server_debug_output"
C2_RPC_GET_AVAILABLE_UI_FUNCTIONS = "c2_rpc_get_ui_functions"
CONTAINER_RPC_GET_FILE = "container_rpc_get_file"
CONTAINER_RPC_REMOVE_FILE = "container_rpc_remove_file"
CONTAINER_RPC_LIST_FILE = "container_rpc_list_file"
CONTAINER_RPC_WRITE_FILE = "container_rpc_write_file"
C2_RPC_HOST_FILE_ROUTING_KEY = "c2_rpc_host_file"
# tr routes
TR_SYNC_ROUTING_KEY = "tr_sync"
TR_RPC_RESYNC_ROUTING_KEY = "tr_rpc_resync"
# webhook routes
EMIT_WEBHOOK_ROUTING_KEY_PREFIX = "emit_webhook"
WEBHOOK_TYPE_NEW_CALLBACK = "new_callback"
WEBHOOK_TYPE_NEW_FEEDBACK = "new_feedback"
WEBHOOK_TYPE_NEW_STARTUP = "new_startup"
WEBHOOK_TYPE_NEW_ALERT = "new_alert"
WEBHOOK_TYPE_NEW_CUSTOM = "new_custom"
# log routes
EMIT_LOG_ROUTING_KEY_PREFIX = "emit_log"
LOG_TYPE_CALLBACK = "new_callback"
LOG_TYPE_CREDENTIAL = "new_credential"
LOG_TYPE_FILE = "new_file"
LOG_TYPE_ARTIFACT = "new_artifact"
LOG_TYPE_TASK = "new_task"
LOG_TYPE_PAYLOAD = "new_payload"
LOG_TYPE_KEYLOG = "new_keylog"
LOG_TYPE_RESPONSE = "new_response"
# CONSUMING_SERVICES_ROUTING_KEY
CONSUMING_CONTAINER_SYNC_ROUTING_KEY   = "consuming_container_sync"
CONSUMING_CONTAINER_RESYNC_ROUTING_KEY = "consuming_container_rpc_resync"
# EVENTING
EVENTING_CUSTOM_FUNCTION_RESPONSE    = "eventing_custom_function_response"
EVENTING_CUSTOM_FUNCTION             = "eventing_custom_function"
EVENTING_CONDITIONAL_CHECK_RESPONSE  = "eventing_conditional_check_response"
EVENTING_CONDITIONAL_CHECK           = "eventing_conditional_check"
EVENTING_TASK_INTERCEPT              = "eventing_task_intercept"
EVENTING_TASK_INTERCEPT_RESPONSE     = "eventing_task_intercept_response"
EVENTING_RESPONSE_INTERCEPT          = "eventing_response_intercept"
EVENTING_RESPONSE_INTERCEPT_RESPONSE = "eventing_response_intercept_response"
# AUTH
AUTH_RPC_GET_IDP_REDIRECT        = "auth_rpc_get_idp_redirect"
AUTH_RPC_PROCESS_IDP_RESPONSE    = "auth_rpc_process_idp_response"
AUTH_RPC_GET_IDP_METADATA        = "auth_rpc_get_idp_metadata"
AUTH_RPC_GET_NONIDP_METADATA     = "auth_rpc_get_nonidp_metadata"
AUTH_RPC_GET_NONIDP_REDIRECT     = "auth_rpc_get_nonidp_redirect"
AUTH_RPC_PROCESS_NONIDP_RESPONSE = "auth_rpc_process_nonidp_response"

CONTAINER_ON_START          = "container_on_start"
CONTAINER_ON_START_RESPONSE = "container_on_start_response"



import mythic_container
from mythic_container.logging import logger
import base64

MYTHIC_RPC_APITOKEN_CREATE    = "mythic_rpc_apitoken_create"


class MythicRPCAPITokenCreateMessage:
    def __init__(self,
                 AgentTaskID: str = None,
                 AgentCallbackID: str = None,
                 PayloadUUID: str = None,
                 OperationID: int = None,
                 **kwargs):
        self.AgentTaskID = AgentTaskID
        self.AgentCallbackID = AgentCallbackID
        self.PayloadUUID = PayloadUUID
        self.OperationID = OperationID
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "agent_task_id": self.AgentTaskID,
            "agentCallback_id": self.AgentCallbackID,
            "payload_uuid": self.PayloadUUID,
            "operation_id": self.OperationID
        }


class MythicRPCAPITokenCreateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 apitoken: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        self.APIToken = apitoken
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCAPITokenCreate(
        msg: MythicRPCAPITokenCreateMessage) -> MythicRPCAPITokenCreateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_APITOKEN_CREATE,
                                                                            body=msg.to_json())
    return MythicRPCAPITokenCreateMessageResponse(**response)



import ujson
from .logging import logger
from .LoggingBase import *


async def new_callback(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        loggingServices = Log.__subclasses__()
        for cls in loggingServices:
            definedLogger = cls()
            if definedLogger.new_callback is not None and callable(definedLogger.new_callback):
                await definedLogger.new_callback(LoggingMessage(**msgDict))
            else:
                logger.error(f"No valid function for new_callback from logger")
    except Exception as e:
        logger.exception(f"Failed to execute new_callback log: {e}")


async def new_credential(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        loggingServices = Log.__subclasses__()
        for cls in loggingServices:
            definedLogger = cls()
            if definedLogger.new_credential is not None and callable(definedLogger.new_credential):
                await definedLogger.new_credential(LoggingMessage(**msgDict))
            else:
                logger.error(f"No valid function for new_credential from logger")
    except Exception as e:
        logger.exception(f"Failed to execute new_credential log: {e}")


async def new_keylog(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        loggingServices = Log.__subclasses__()
        for cls in loggingServices:
            definedLogger = cls()
            if definedLogger.new_keylog is not None and callable(definedLogger.new_keylog):
                await definedLogger.new_keylog(LoggingMessage(**msgDict))
            else:
                logger.error(f"No valid function for new_keylog from logger")
    except Exception as e:
        logger.exception(f"Failed to execute new_keylog log: {e}")


async def new_file(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        loggingServices = Log.__subclasses__()
        for cls in loggingServices:
            definedLogger = cls()
            if definedLogger.new_file is not None and callable(definedLogger.new_file):
                await definedLogger.new_file(LoggingMessage(**msgDict))
            else:
                logger.error(f"No valid function for new_file from logger")
    except Exception as e:
        logger.exception(f"Failed to execute new_file log: {e}")


async def new_payload(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        loggingServices = Log.__subclasses__()
        for cls in loggingServices:
            definedLogger = cls()
            if definedLogger.new_payload is not None and callable(definedLogger.new_payload):
                await definedLogger.new_payload(LoggingMessage(**msgDict))
            else:
                logger.error(f"No valid function for new_payload from logger")
    except Exception as e:
        logger.exception(f"Failed to execute new_payload log: {e}")


async def new_artifact(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        loggingServices = Log.__subclasses__()
        for cls in loggingServices:
            definedLogger = cls()
            if definedLogger.new_artifact is not None and callable(definedLogger.new_artifact):
                await definedLogger.new_artifact(LoggingMessage(**msgDict))
            else:
                logger.error(f"No valid function for new_artifact from logger")
    except Exception as e:
        logger.exception(f"Failed to execute new_artifact log: {e}")


async def new_task(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        loggingServices = Log.__subclasses__()
        for cls in loggingServices:
            definedLogger = cls()
            if definedLogger.new_task is not None and callable(definedLogger.new_task):
                await definedLogger.new_task(LoggingMessage(**msgDict))
            else:
                logger.error(f"No valid function for new_task from logger")
    except Exception as e:
        logger.exception(f"Failed to execute new_task log: {e}")


async def new_response(msg: bytes) -> None:
    try:
        msgDict = ujson.loads(msg)
        loggingServices = Log.__subclasses__()
        for cls in loggingServices:
            definedLogger = cls()
            if definedLogger.new_response is not None and callable(definedLogger.new_response):
                await definedLogger.new_response(LoggingMessage(**msgDict))
            else:
                logger.error(f"No valid function for new_response from logger")
    except Exception as e:
        logger.exception(f"Failed to execute new_response log: {e}")



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_TASK_CREATE_SUBTASK_GROUP = "mythic_rpc_task_create_group"


class MythicRPCTaskCreateSubtaskGroupTasks:
    def __init__(self,
                 SubtaskCallbackFunction: str = None,
                 CommandName: str = None,
                 Params: str = None,
                 ParameterGroupName: str = None,
                 Token: int = None,
                 **kwargs):
        self.SubtaskCallbackFunction = SubtaskCallbackFunction
        self.CommandName = CommandName
        self.Params = Params
        self.ParameterGroupName = ParameterGroupName
        self.Token = Token
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")
    def to_json(self):
        return {
            "subtask_callback_function": self.SubtaskCallbackFunction,
            "command_name": self.CommandName,
            "params": self.Params,
            "parameter_group_name": self.ParameterGroupName,
            "token": self.Token
        }

class MythicRPCTaskCreateSubtaskGroupMessage:
    def __init__(self,
                 TaskID: int,
                 GroupName: str,
                 GroupCallbackFunction: str = None,
                 Tasks: list[MythicRPCTaskCreateSubtaskGroupTasks] = None,
                 **kwargs):
        self.TaskID = TaskID
        self.GroupName = GroupName
        self.GroupCallbackFunction = GroupCallbackFunction
        self.Tasks = Tasks
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "group_name": self.GroupName,
            "group_callback_function": self.GroupCallbackFunction,
            "tasks": [x.to_json() for x in self.Tasks]
        }


class MythicRPCTaskCreateSubtaskGroupMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 task_ids: [int] = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.TaskIDs = task_ids
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCTaskCreateSubtaskGroup(
        msg: MythicRPCTaskCreateSubtaskGroupMessage) -> MythicRPCTaskCreateSubtaskGroupMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_TASK_CREATE_SUBTASK_GROUP,
                                                                            body=msg.to_json())
    return MythicRPCTaskCreateSubtaskGroupMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger
import time

MYTHIC_RPC_FILE_SEARCH = "mythic_rpc_file_search"


class MythicRPCFileSearchMessage:
    def __init__(self,
                 TaskID: int = None,
                 CallbackID: int = None,
                 LimitByCallback: bool = False,
                 MaxResults: int = -1,
                 Filename: str = None,
                 IsScreenshot: bool = False,
                 IsDownloadFromAgent: bool = False,
                 IsPayload: bool = False,
                 AgentFileID: str = None,
                 Comment: str = "",
                 **kwargs):
        self.TaskID = TaskID
        self.CallbackID = CallbackID
        self.LimitByCallback = LimitByCallback
        self.MaxResults = MaxResults
        self.Filename = Filename
        self.IsScreenshot = IsScreenshot
        self.IsDownloadFromAgent = IsDownloadFromAgent
        self.Comment = Comment
        self.IsPayload = IsPayload
        self.AgentFileID = AgentFileID
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "callback_id": self.CallbackID,
            "limit_by_callback": self.LimitByCallback,
            "max_results": self.MaxResults,
            "filename": self.Filename,
            "is_screenshot": self.IsScreenshot,
            "is_download_from_agent": self.IsDownloadFromAgent,
            "is_payload": self.IsPayload,
            "file_id": self.AgentFileID,
            "comment": self.Comment
        }


class FileData:
    def __init__(self,
                 agent_file_id: str = None,
                 filename: str = None,
                 comment: str = None,
                 is_payload: bool = None,
                 is_download_from_agent: bool = None,
                 is_screenshot: bool = None,
                 full_remote_path: str = None,
                 host: str = None,
                 task_id: int = None,
                 md5: str = None,
                 sha1: str = None,
                 timestamp: time = None,
                 cmd: str = None,
                 tags: list[str] = None,
                 complete: bool = None,
                 **kwargs):
        self.AgentFileId = agent_file_id
        self.Filename = filename
        self.Comment = comment
        self.IsPayload = is_payload
        self.IsDownloadFromAgent = is_download_from_agent
        self.IsScreenshot = is_screenshot
        self.FullRemotePath = full_remote_path
        self.Host = host
        self.TaskID = task_id
        self.Md5 = md5
        self.Sha1 = sha1
        self.Timestamp = timestamp
        self.Command = cmd
        self.Tags = tags
        self.Complete = complete
        for k, v in kwargs.items():
            logger.info(f"unknown kwarg {k} with value {v}")
    def to_json(self):
        return {
            "agent_file_id": self.AgentFileId,
            "filename": self.Filename,
            "comment": self.Comment,
            "is_payload": self.IsPayload,
            "is_download_from_agent": self.IsDownloadFromAgent,
            "is_screenshot": self.IsScreenshot,
            "full_remote_path": self.FullRemotePath,
            "host": self.Host,
            "task_id": self.TaskID,
            "md5": self.Md5,
            "sha1": self.Sha1,
            "timestamp": self.Timestamp,
            "cmd": self.Command,
            "tags": self.Tags,
            "complete": self.Complete
        }
class MythicRPCFileSearchMessageResponse:
    Success: bool
    Error: str
    Files: list[FileData]

    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 files: list[dict] = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.Files = [FileData(**x) for x in files] if files is not None else []
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCFileSearch(
        msg: MythicRPCFileSearchMessage) -> MythicRPCFileSearchMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_FILE_SEARCH,
                                                                            body=msg.to_json())
    finalResponse = MythicRPCFileSearchMessageResponse(**response)
    return finalResponse



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class PushC2MessageFromAgent(_message.Message):
    __slots__ = ["AgentDisconnected", "Base64Message", "C2ProfileName", "Message", "OuterUUID", "RemoteIP", "TrackingID"]
    AGENTDISCONNECTED_FIELD_NUMBER: _ClassVar[int]
    AgentDisconnected: bool
    BASE64MESSAGE_FIELD_NUMBER: _ClassVar[int]
    Base64Message: bytes
    C2PROFILENAME_FIELD_NUMBER: _ClassVar[int]
    C2ProfileName: str
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    Message: bytes
    OUTERUUID_FIELD_NUMBER: _ClassVar[int]
    OuterUUID: str
    REMOTEIP_FIELD_NUMBER: _ClassVar[int]
    RemoteIP: str
    TRACKINGID_FIELD_NUMBER: _ClassVar[int]
    TrackingID: str
    def __init__(self, C2ProfileName: _Optional[str] = ..., RemoteIP: _Optional[str] = ..., Message: _Optional[bytes] = ..., OuterUUID: _Optional[str] = ..., Base64Message: _Optional[bytes] = ..., TrackingID: _Optional[str] = ..., AgentDisconnected: bool = ...) -> None: ...

class PushC2MessageFromMythic(_message.Message):
    __slots__ = ["Error", "Message", "Success", "TrackingID"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    Error: str
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    Message: bytes
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    Success: bool
    TRACKINGID_FIELD_NUMBER: _ClassVar[int]
    TrackingID: str
    def __init__(self, Success: bool = ..., Error: _Optional[str] = ..., Message: _Optional[bytes] = ..., TrackingID: _Optional[str] = ...) -> None: ...



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_TASK_CREATE = "mythic_rpc_task_create"


class MythicRPCTaskCreateMessage:
    def __init__(self,
                 AgentCallbackID: str,
                 CommandName: str = None,
                 Params: str = None,
                 ParameterGroupName: str = None,
                 Token: int = None,
                 **kwargs):
        self.AgentCallbackID = AgentCallbackID
        self.CommandName = CommandName
        self.Params = Params
        self.ParameterGroupName = ParameterGroupName
        self.Token = Token
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "agent_callback_id": self.AgentCallbackID,
            "command_name": self.CommandName,
            "params": self.Params,
            "parameter_group_name": self.ParameterGroupName,
            "token": self.Token
        }


class MythicRPCTaskCreateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 task_id: int = None,
                 task_display_id: int = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.TaskID = task_id
        self.TaskDisplayID = task_display_id
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "task_id": self.TaskID,
            "task_display_id": self.TaskDisplayID,
        }


async def SendMythicRPCTaskCreate(
        msg: MythicRPCTaskCreateMessage) -> MythicRPCTaskCreateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_TASK_CREATE,
                                                                            body=msg.to_json())
    return MythicRPCTaskCreateMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_PROXY_STOP = "mythic_rpc_proxy_stop"


class MythicRPCProxyStopMessage:
    def __init__(self,
                 TaskID: int,
                 Port: int,
                 PortType: str,
                 **kwargs):
        self.TaskID = TaskID
        self.Port = Port
        self.PortType = PortType
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "port": self.Port,
            "port_type": self.PortType
        }


class MythicRPCProxyStopMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 local_port: int = 0,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.LocalPort = local_port
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCProxyStopCommand(
        msg: MythicRPCProxyStopMessage) -> MythicRPCProxyStopMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_PROXY_STOP,
                                                                            body=msg.to_json())
    return MythicRPCProxyStopMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_PROCESS_CREATE = "mythic_rpc_process_create"

class MythicRPCProcessCreateData:
    def __init__(self,
                 Host: str = None,
                 ProcessID: int = None,
                 ParentProcessID: int = None,
                 Architecture: str = None,
                 BinPath: str = None,
                 Name: str = None,
                 User: str = None,
                 CommandLine: str = None,
                 IntegrityLevel: int = None,
                 StartTime: int = None,
                 Description: str = None,
                 Signer: str = None,
                 ProtectionProcessLevel: int = None,
                 **kwargs):
        self.Host = Host
        self.ProcessID = ProcessID
        self.ParentProcessID = ParentProcessID
        self.Architecture = Architecture
        self.BinPath = BinPath
        self.Name = Name
        self.User = User
        self.CommandLine = CommandLine
        self.IntegrityLevel = IntegrityLevel
        self.StartTime = StartTime
        self.Description = Description
        self.Signer = Signer
        self.ProtectionProcessLevel = ProtectionProcessLevel
        self.other = kwargs

    def to_json(self):
        return {
            "host": self.Host,
            "process_id": self.ProcessID,
            "parent_process_id": self.ParentProcessID,
            "architecture": self.Architecture,
            "bin_path": self.BinPath,
            "name": self.Name,
            "user": self.User,
            "command_line": self.CommandLine,
            "integrity_level": self.IntegrityLevel,
            "start_time": self.StartTime,
            "description": self.Description,
            "signer": self.Signer,
            "protected_process_level": self.ProtectionProcessLevel,
            **self.other
        }


class MythicRPCProcessesCreateMessage:
    def __init__(self,
                 TaskID: int,
                 Processes: list[MythicRPCProcessCreateData] = None,
                 **kwargs):
        self.TaskID = TaskID
        self.Processes = Processes
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "processes": [x.to_json() for x in self.Processes]
        }


class MythicRPCProcessesCreateMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCProcessCreate(
        msg: MythicRPCProcessesCreateMessage) -> MythicRPCProcessesCreateMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_PROCESS_CREATE,
                                                                            body=msg.to_json())
    return MythicRPCProcessesCreateMessageResponse(**response)



from collections.abc import Callable, Awaitable
from typing import Union

import mythic_container
from .logging import logger
import base64
import json
from .MythicCommandBase import PTTaskMessageTaskData
from .SharedClasses import ContainerOnStartMessage, ContainerOnStartMessageResponse


class GetIDPMetadataMessage:
    def __init__(self,
                 container_name: str = "",
                 server_name: str = "",
                 idp_name: str = "",
                 **kwargs):
        self.ContainerName = container_name
        self.ServerName = server_name
        self.IDPName = idp_name

    def to_json(self):
        return {
            "container_name": self.ContainerName,
            "server_name": self.ServerName,
            "idp_name": self.IDPName
        }


class GetIDPMetadataMessageResponse:
    def __init__(self,
                 Success: bool = True,
                 Error: str = "",
                 Metadata: str = "",
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Metadata = Metadata

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "metadata": self.Metadata
        }


class GetIDPRedirectMessage:
    def __init__(self,
                 container_name: str = "",
                 server_name: str = "",
                 idp_name: str = "",
                 request_url: str = "",
                 request_headers: dict = {},
                 request_cookies: dict = {},
                 request_query: dict = {}):
        self.ContainerName = container_name
        self.ServerName = server_name
        self.IDPName = idp_name
        self.RequestURL = request_url
        self.RequestHeaders = request_headers
        self.RequestCookies = request_cookies
        self.RequestQuery = request_query

    def to_json(self):
        return {
            "container_name": self.ContainerName,
            "server_name": self.ServerName,
            "idp_name": self.IDPName,
            "request_url": self.RequestURL,
            "request_headers": self.RequestHeaders,
            "request_cookies": self.RequestCookies,
            "request_query": self.RequestQuery
        }


class GetIDPRedirectMessageResponse:
    def __init__(self,
                 Success: bool = True,
                 Error: str = "",
                 RedirectURL: str = "",
                 RedirectHeaders: dict = {},
                 RedirectCookies: dict = {}):
        self.Success = Success
        self.Error = Error
        self.RedirectURL = RedirectURL
        self.RedirectHeaders = RedirectHeaders
        self.RedirectCookies = RedirectCookies

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "redirect_url": self.RedirectURL,
            "redirect_headers": self.RedirectHeaders,
            "redirect_cookies": self.RedirectCookies
        }


class GetNonIDPMetadataMessage:
    def __init__(self,
                 container_name: str = "",
                 server_name: str = "",
                 nonidp_name: str = "",
                 **kwargs):
        self.ContainerName = container_name
        self.ServerName = server_name
        self.NonIDPName = nonidp_name

    def to_json(self):
        return {
            "container_name": self.ContainerName,
            "server_name": self.ServerName,
            "nonidp_name": self.NonIDPName
        }


class GetNonIDPMetadataMessageResponse:
    def __init__(self,
                 Success: bool = True,
                 Error: str = "",
                 Metadata: str = "",
                 **kwargs):
        self.Success = Success
        self.Error = Error
        self.Metadata = Metadata

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "metadata": self.Metadata
        }


class GetNonIDPRedirectMessage:
    def __init__(self,
                 container_name: str = "",
                 server_name: str = "",
                 nonidp_name: str = ""):
        self.ContainerName = container_name
        self.ServerName = server_name
        self.NonIDPName = nonidp_name

    def to_json(self):
        return {
            "container_name": self.ContainerName,
            "server_name": self.ServerName,
            "nonidp_name": self.NonIDPName,
        }


class GetNonIDPRedirectMessageResponse:
    def __init__(self,
                 Success: bool = True,
                 Error: str = "",
                 RequestFields: [str] = []):
        self.Success = Success
        self.Error = Error
        self.RequestFields = RequestFields

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "request_fields": self.RequestFields,
        }


class ProcessIDPResponseMessage:
    def __init__(self,
                 container_name: str = "",
                 server_name: str = "",
                 idp_name: str = "",
                 request_url: str = "",
                 request_headers: dict = {},
                 request_cookies: dict = {},
                 request_query: dict = {},
                 request_body: str = ""):
        self.ContainerName = container_name
        self.ServerName = server_name
        self.IDPName = idp_name
        self.RequestURL = request_url
        self.RequestHeaders = request_headers
        self.RequestCookies = request_cookies
        self.RequestQuery = request_query
        self.RequestBody = request_body

    def to_json(self):
        return {
            "container_name": self.ContainerName,
            "server_name": self.ServerName,
            "idp_name": self.IDPName,
            "request_url": self.RequestURL,
            "request_headers": self.RequestHeaders,
            "request_cookies": self.RequestCookies,
            "request_query": self.RequestQuery,
            "request_body": self.RequestBody
        }


class ProcessIDPResponseMessageResponse:
    def __init__(self,
                 SuccessfulAuthentication: bool = False,
                 Error: str = "",
                 Email: str = "", ):
        self.SuccessfulAuthentication = SuccessfulAuthentication
        self.Error = Error
        self.Email = Email

    def to_json(self):
        return {
            "successful_authentication": self.SuccessfulAuthentication,
            "error": self.Error,
            "email": self.Email,
        }


class ProcessNonIDPResponseMessage:
    def __init__(self,
                 container_name: str = "",
                 server_name: str = "",
                 idp_name: str = "",
                 request_values: dict = {}):
        self.ContainerName = container_name
        self.ServerName = server_name
        self.IDPName = idp_name
        self.RequestValues = request_values

    def to_json(self):
        return {
            "container_name": self.ContainerName,
            "server_name": self.ServerName,
            "idp_name": self.IDPName,
            "request_values": self.RequestValues
        }


class ProcessNonIDPResponseMessageResponse:
    def __init__(self,
                 SuccessfulAuthentication: bool = False,
                 Error: str = "",
                 Email: str = "", ):
        self.SuccessfulAuthentication = SuccessfulAuthentication
        self.Error = Error
        self.Email = Email

    def to_json(self):
        return {
            "successful_authentication": self.SuccessfulAuthentication,
            "error": self.Error,
            "email": self.Email,
        }


class Auth:
    """Auth definition class for performing custom auth within Mythic


    Attributes:
        name:
            name for this auth service that appears in the Mythic UI
        description:
            description about what this auth service provides that appears in the Mythic UI
        idp_services:
            list of names of IDP services you want to expose into the Mythic UI for users to select
        non_idp_services:
            list of names of Non-IDP auth options you want to expose into the Mythic UI for users to select

    Functions:

    """
    name: str = ""
    description: str = ""
    idp_services: [str] = []
    non_idp_services: [str] = []

    get_idp_metadata: Callable[[GetIDPMetadataMessage], Awaitable[GetIDPMetadataMessageResponse]] = None
    get_idp_redirect: Callable[[GetIDPRedirectMessage], Awaitable[GetIDPRedirectMessageResponse]] = None
    process_idp_response: Callable[[ProcessIDPResponseMessage], Awaitable[ProcessIDPResponseMessageResponse]] = None
    get_non_idp_metadata: Callable[[GetNonIDPMetadataMessage], Awaitable[GetNonIDPMetadataMessageResponse]] = None
    get_non_idp_redirect: Callable[[GetNonIDPRedirectMessage], Awaitable[GetNonIDPRedirectMessageResponse]] = None
    process_non_idp_response: Callable[[ProcessNonIDPResponseMessage], Awaitable[ProcessNonIDPResponseMessageResponse]] = None

    async def on_container_start(self, message: ContainerOnStartMessage) -> ContainerOnStartMessageResponse:
        return ContainerOnStartMessageResponse(ContainerName=self.name)

    def get_sync_message(self):
        idp_subs = [json.dumps({"name": x, "type": "idp"}) for x in self.idp_services]
        non_idp_subs = [json.dumps({"name": x, "type": "nonidp"}) for x in self.non_idp_services]
        return {
            "name": self.name,
            "type": "auth",
            "description": self.description,
            "subscriptions": idp_subs + non_idp_subs
        }


authServices: dict[str, Auth] = {}



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import mythic_container.grpc.pushC2GRPC_pb2 as pushC2GRPC__pb2


class PushC2Stub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StartPushC2Streaming = channel.stream_stream(
                '/pushC2Services.PushC2/StartPushC2Streaming',
                request_serializer=pushC2GRPC__pb2.PushC2MessageFromAgent.SerializeToString,
                response_deserializer=pushC2GRPC__pb2.PushC2MessageFromMythic.FromString,
                )
        self.StartPushC2StreamingOneToMany = channel.stream_stream(
                '/pushC2Services.PushC2/StartPushC2StreamingOneToMany',
                request_serializer=pushC2GRPC__pb2.PushC2MessageFromAgent.SerializeToString,
                response_deserializer=pushC2GRPC__pb2.PushC2MessageFromMythic.FromString,
                )


class PushC2Servicer(object):
    """Missing associated documentation comment in .proto file."""

    def StartPushC2Streaming(self, request_iterator, context):
        """rpc function(messageFromClient) messageToClient
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StartPushC2StreamingOneToMany(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PushC2Servicer_to_server(servicer, server):
    rpc_method_handlers = {
            'StartPushC2Streaming': grpc.stream_stream_rpc_method_handler(
                    servicer.StartPushC2Streaming,
                    request_deserializer=pushC2GRPC__pb2.PushC2MessageFromAgent.FromString,
                    response_serializer=pushC2GRPC__pb2.PushC2MessageFromMythic.SerializeToString,
            ),
            'StartPushC2StreamingOneToMany': grpc.stream_stream_rpc_method_handler(
                    servicer.StartPushC2StreamingOneToMany,
                    request_deserializer=pushC2GRPC__pb2.PushC2MessageFromAgent.FromString,
                    response_serializer=pushC2GRPC__pb2.PushC2MessageFromMythic.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'pushC2Services.PushC2', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class PushC2(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def StartPushC2Streaming(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/pushC2Services.PushC2/StartPushC2Streaming',
            pushC2GRPC__pb2.PushC2MessageFromAgent.SerializeToString,
            pushC2GRPC__pb2.PushC2MessageFromMythic.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StartPushC2StreamingOneToMany(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/pushC2Services.PushC2/StartPushC2StreamingOneToMany',
            pushC2GRPC__pb2.PushC2MessageFromAgent.SerializeToString,
            pushC2GRPC__pb2.PushC2MessageFromMythic.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_PROXY_START = "mythic_rpc_proxy_start"

CALLBACK_PORT_TYPE_SOCKS = "socks"
CALLBACK_PORT_TYPE_RPORTFWD = "rpfwd"
CALLBACK_PORT_TYPE_INTERACTIVE = "interactive"


class MythicRPCProxyStartMessage:
    """
    Start a new proxy session with Mythic.
    For CALLBACK_PORT_TYPE_SOCKS and CALLBACK_PORT_TYPE_INTERACTIVE:
        if LocalPort is 0, then Mythic will assign the next available port
    For PortType == CALLBACK_PORT_TYPE_SOCKS:
        LocalPort is used to open up a port on Mythic
    For PortType == CALLBACK_PORT_TYPE_RPORTFWD:
        LocalPort is used to open up a port on the host where the agent is running
        RemoteIP:RemotePort is used for Mythic to connect to when LocalPort gets a connection
    For PortType == CALLBACK_PORT_TYPE_INTERACTIVE:
        LocalPort is used to open up a port on Mythic
    """

    def __init__(self,
                 TaskID: int,
                 PortType: str,
                 LocalPort: int = None,
                 RemotePort: int = None,
                 RemoteIP: str = None,
                 **kwargs):
        self.TaskID = TaskID
        self.LocalPort = LocalPort
        self.RemotePort = RemotePort
        self.RemoteIP = RemoteIP
        self.PortType = PortType
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "task_id": self.TaskID,
            "local_port": self.LocalPort,
            "remote_port": self.RemotePort,
            "remote_ip": self.RemoteIP,
            "port_type": self.PortType
        }


class MythicRPCProxyStartMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 local_port: int = 0,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.LocalPort = local_port
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCProxyStartCommand(
        msg: MythicRPCProxyStartMessage) -> MythicRPCProxyStartMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_PROXY_START,
                                                                            body=msg.to_json())
    return MythicRPCProxyStartMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger

MYTHIC_RPC_PAYLOAD_UPDATE_BUILD_STEP = "mythic_rpc_payload_update_build_step"


class MythicRPCPayloadUpdateBuildStepMessage:
    def __init__(self,
                 PayloadUUID: str,
                 StepName: str = "",
                 StepStdout: str = "",
                 StepStderr: str = "",
                 StepSuccess: bool = False,
                 StepSkip: bool = False,
                 **kwargs):
        self.PayloadUUID = PayloadUUID
        self.StepName = StepName
        self.StepStdout = StepStdout
        self.StepStderr = StepStderr
        self.StepSuccess = StepSuccess
        self.StepSkip = StepSkip
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "payload_uuid": self.PayloadUUID,
            "step_name": self.StepName,
            "step_stdout": self.StepStdout,
            "step_stderr": self.StepStderr,
            "step_success": self.StepSuccess,
            "step_skip": self.StepSkip
        }


class MythicRPCPayloadUpdateBuildStepMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCPayloadUpdatebuildStep(
        msg: MythicRPCPayloadUpdateBuildStepMessage) -> MythicRPCPayloadUpdateBuildStepMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_PAYLOAD_UPDATE_BUILD_STEP,
                                                                            body=msg.to_json())
    return MythicRPCPayloadUpdateBuildStepMessageResponse(**response)


import mythic_container
from mythic_container.logging import logger
import base64

MYTHIC_RPC_CALLBACK_NEXT_CHECKIN_RANGE = "mythic_rpc_callback_next_checkin_range"


class MythicRPCCallbackNextCheckinRangeMessage:
    def __init__(self,
                 SleepInterval: int = None,
                 SleepJitter: int = None,
                 LastCheckin: str = None,
                 **kwargs):
        self.SleepInterval = SleepInterval
        self.SleepJitter = SleepJitter
        self.LastCheckin = LastCheckin
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "sleep_interval": self.SleepInterval,
            "sleep_jitter": self.SleepJitter,
            "last_checkin": self.LastCheckin,
        }


class MythicRPCCallbackNextCheckinRangeMessageResponse:
    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 min: str = "",
                 max: str = "",
                 **kwargs):
        self.Success = success
        self.Error = error
        self.Min = min
        self.Max = max
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "success": self.Success,
            "error": self.Error,
            "min": self.Min,
            "max": self.Max,
        }


async def SendMythicRPCCallbackNextCheckinRange(
        msg: MythicRPCCallbackNextCheckinRangeMessage) -> MythicRPCCallbackNextCheckinRangeMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(
        queue=MYTHIC_RPC_CALLBACK_NEXT_CHECKIN_RANGE,
        body=msg.to_json())
    return MythicRPCCallbackNextCheckinRangeMessageResponse(**response)



import mythic_container
from mythic_container.logging import logger
from mythic_container.MythicGoRPC.send_mythic_rpc_callback_search import MythicRPCCallbackSearchMessageResult
import time

MYTHIC_RPC_CALLBACK_EDGE_SEARCH = "mythic_rpc_callback_edge_search"


class MythicRPCCallbackEdgeSearchMessage:
    def __init__(self,
                 AgentCallbackUUID: str = None,
                 AgentCallbackID: int = None,
                 SearchC2ProfileName: str = None,
                 SearchActiveEdgesOnly: bool = False,
                 **kwargs):
        self.AgentCallbackID = AgentCallbackID
        self.AgentCallbackUUID = AgentCallbackUUID
        self.SearchC2ProfileName = SearchC2ProfileName
        self.SearchActiveEdgesOnly = SearchActiveEdgesOnly
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "agent_callback_id": self.AgentCallbackUUID,
            "callback_id": self.AgentCallbackID,
            "search_c2_profile_name": self.SearchC2ProfileName,
            "search_active_edges_only": self.SearchActiveEdgesOnly,
        }


class MythicRPCCallbackEdgeSearchMessageResult:
    def __init__(self,
                 source: MythicRPCCallbackSearchMessageResult,
                 destination: MythicRPCCallbackSearchMessageResult,
                 c2profile: str,
                 id: int = None,
                 start_timestamp: str = None,
                 end_timestamp: str = None,
                 **kwargs):
        self.ID = id
        self.StartTimestamp = start_timestamp
        self.EndTimestamp = end_timestamp
        self.Source = source
        self.Destination = destination
        self.C2ProfileName = c2profile

        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")

    def to_json(self):
        return {
            "id": self.ID,
            "start_timestamp": self.StartTimestamp,
            "end_timestamp": self.EndTimestamp,
            "source": self.Source.to_json(),
            "destination": self.Destination.to_json(),
            "c2profile": self.C2ProfileName,
        }


class MythicRPCCallbackEdgeSearchMessageResponse:
    Results: list[MythicRPCCallbackEdgeSearchMessageResult]

    def __init__(self,
                 success: bool = False,
                 error: str = "",
                 results: list[dict] = None,
                 **kwargs):
        self.Success = success
        self.Error = error
        self.Results = [MythicRPCCallbackEdgeSearchMessageResult(**x) for x in results] if results is not None else []
        for k, v in kwargs.items():
            logger.info(f"Unknown kwarg {k} - {v}")


async def SendMythicRPCCallbackEdgeSearch(
        msg: MythicRPCCallbackEdgeSearchMessage) -> MythicRPCCallbackEdgeSearchMessageResponse:
    response = await mythic_container.RabbitmqConnection.SendRPCDictMessage(queue=MYTHIC_RPC_CALLBACK_EDGE_SEARCH,
                                                                            body=msg.to_json())
    return MythicRPCCallbackEdgeSearchMessageResponse(**response)
